from ophyd import (EpicsSignal, EpicsSignalRO)

from .. import (RecordBase, _register_record_type,
                FieldComponent as Cpt)


@_register_record_type('scanparm')
class ScanparmRecord(RecordBase):
    alarm_status = Cpt(EpicsSignalRO, '.STAT')
    code_version = Cpt(EpicsSignalRO, '.VERS')
    last_stepsize = Cpt(EpicsSignalRO, '.LSTP')
    maxpts = Cpt(EpicsSignalRO, '.MP')
    scanactive = Cpt(EpicsSignalRO, '.ACT')
    stepsize = Cpt(EpicsSignalRO, '.STEP')

    # - common
    after_outlink = Cpt(EpicsSignalRO, '.OAFT$', string=True)
    acquire_time_outlink = Cpt(EpicsSignalRO, '.OAQT$', string=True)
    ar_outlink = Cpt(EpicsSignalRO, '.OAR$', string=True)
    after = Cpt(EpicsSignal, '.AFT')
    d1pv_outlink = Cpt(EpicsSignalRO, '.ODPV$', string=True)
    detpvname = Cpt(EpicsSignal, '.DPV$', string=True)
    ep_outlink = Cpt(EpicsSignalRO, '.OEP$', string=True)
    go_outlink = Cpt(EpicsSignalRO, '.OGO$', string=True)
    inlink = Cpt(EpicsSignalRO, '.IACT$', string=True)
    load_outlink = Cpt(EpicsSignalRO, '.OLOAD$', string=True)
    mp_inlink = Cpt(EpicsSignalRO, '.IMP$', string=True)
    np_outlink = Cpt(EpicsSignalRO, '.ONP$', string=True)
    p1pv_outlink = Cpt(EpicsSignalRO, '.OPPV$', string=True)
    pre_write_outlink = Cpt(EpicsSignalRO, '.OPRE$', string=True)
    positionerpvname = Cpt(EpicsSignal, '.PPV$', string=True)
    r1pv_outlink = Cpt(EpicsSignalRO, '.ORPV$', string=True)
    readbackpvname = Cpt(EpicsSignal, '.RPV$', string=True)
    sc_outlink = Cpt(EpicsSignalRO, '.OSC$', string=True)
    sm_outlink = Cpt(EpicsSignalRO, '.OSM$', string=True)
    sp_outlink = Cpt(EpicsSignalRO, '.OSP$', string=True)
    stepmode = Cpt(EpicsSignal, '.SM')
    t1pv_outlink = Cpt(EpicsSignalRO, '.OTPV$', string=True)
    trigpvname = Cpt(EpicsSignal, '.TPV$', string=True)
    absrel = Cpt(EpicsSignal, '.AR')

    # - display
    acquire_time = Cpt(EpicsSignal, '.AQT')
    display_precision = Cpt(EpicsSignal, '.PREC')
    endpos = Cpt(EpicsSignal, '.EP')
    go = Cpt(EpicsSignal, '.GO')
    load = Cpt(EpicsSignal, '.LOAD')
    pre_write_command = Cpt(EpicsSignal, '.PRE')
    startcmd = Cpt(EpicsSignal, '.SC')
    startpos = Cpt(EpicsSignal, '.SP')
    npts = Cpt(EpicsSignal, '.NP')
